@@ -6,7 +6,8 @@ use warnings;
 use 5.010;
 
 use List::Util qw(max);
-use List::MoreUtils;
+#use List::MoreUtils;
+use List::MoreUtils qw{firstidx} ;
 use POSIX qw(strftime);
 use Carp;
 use DBI;
@@ -16,17 +17,24 @@ use IO::File ;
 use File::Basename;
 use Try::Tiny;
 
+use VSGDR::UnitTest::TestSet::Test;
+use VSGDR::UnitTest::TestSet::Test::TestCondition;
+use VSGDR::UnitTest::TestSet::Representation;
+use VSGDR::UnitTest::TestSet::Resx;
+use File::Basename;
+
+
 =head1 NAME
 
 VSGDR::TestScriptGen - Unit test script support package for SSDT unit tests, Ded MedVed.
 
 =head1 VERSION
 
-Version 0.12
+Version 0.13
 
 =cut
 
-our $VERSION = '0.12';
+our $VERSION = '0.13';
 
 
 sub databaseName {
@@ -151,13 +159,71 @@ sub generateScripts {
     my $dbh             = shift ;
     my $dbh_typeinfo    = shift ;
     my $dirs            = shift ;
+    my $file            = shift ;
     my $runChecks       = shift ;
 
     croak "bad arg dbh"             unless defined $dbh;
     croak "bad arg dbh_typeinfo"    unless defined $dbh_typeinfo;
     croak "bad arg dirs"            unless defined $dirs;
+    #croak "bad arg file"            unless defined $file;
     croak "bad arg runChecks"       unless defined $runChecks;
     
+    my $testSet = undef;
+    if ( defined $file ) {
+
+        my %ValidParserMakeArgs = ( vb  => "NET::VB"
+                                , cs  => "NET::CS"
+                                , xls => "XLS"
+                                , xml => "XML"
+                                ) ;
+        my %ValidParserMakeArgs2 = ( vb  => "NET2::VB"
+                                , cs  => "NET2::CS"
+                                ) ;                          
+                                
+        #my @validSuffixes       = keys %ValidParserMakeArgs ;
+        my @validSuffixes       = map { '.'.$_ } keys %ValidParserMakeArgs ;
+        
+        my $infile = $file;
+        
+        my($infname, $directories, $insfx)      = fileparse($infile , @validSuffixes);
+        croak 'Invalid input file'   unless defined $insfx ;
+        $insfx        = lc $insfx ;
+        $insfx        = substr $insfx,1;
+        
+        ### Validate parameters
+        die 'Invalid input file'  unless exists $ValidParserMakeArgs{$insfx} ;
+        
+        ### Build parsers
+        
+        my %Parsers            = () ;
+        $Parsers{${insfx}}     = VSGDR::UnitTest::TestSet::Representation->make( { TYPE => $ValidParserMakeArgs{${insfx}} } );
+        # if input is in a .net language, add in a .net2 parser to the list
+        if ( firstidx { $_ eq ${insfx} } ['cs','vb']  != -1 ) {
+            $Parsers{"${insfx}2"}  = VSGDR::UnitTest::TestSet::Representation->make( { TYPE => $ValidParserMakeArgs2{${insfx}} } );
+        }
+        
+        ### Deserialise tests 
+        eval {
+            $testSet         = $Parsers{$insfx}->deserialise($infile);
+            } ;
+        if ( not defined $testSet ) {
+            if ( exists $Parsers{"${insfx}2"}) {
+                eval {
+                    $testSet     = $Parsers{"${insfx}2"}->deserialise($infile);
+                    }
+            }            
+            else {
+                croak 'Parsing failed.'; 
+            }
+        }
+        
+    }
+    my @existingTests = () ;
+    if (defined $testSet) {
+        @existingTests = map {$_->testName()}  @{$testSet->tests()};
+    }
+    
+    
     my $database        = databaseName($dbh);
 
     no warnings;
@@ -171,48 +237,58 @@ sub generateScripts {
     my $execs           = ExecSp($dbh) ;
 
 #warn Dumper     $widest_column_name_padding;
+
     foreach my $exec (@$execs) {
         
-        my $file = $$exec[0];
+        my $ofile = $$exec[0];
         
-        my $checkText    = "";
-        my $receivingTable = "" ; 
-
-        if ( $runChecks ) {
-        
-            $checkText    = CheckForExceptions($dbh, $dbh_typeinfo, $$exec[0], $userName, $date, $$exec[1],$$exec[2] ) ;
-
-            my $resultsTable = undef ;
-            if ( ! defined $checkText || $checkText eq q() ) {
-                $resultsTable = CheckForResults($dbh, $dbh_typeinfo, $$exec[0], $userName, $date, $$exec[1],$$exec[2] ) ;
-            }
-#warn Dumper "--------------------------";            
-#warn Dumper $resultsTable;
-#warn Dumper scalar @$resultsTable ;
-#warn Dumper @{$resultsTable->[0]};
-            if (defined $resultsTable && scalar @$resultsTable eq 1  && scalar @{$resultsTable->[0]} gt 0 ) {
-                $receivingTable = do { local $"= "\n\t,\t\t" ; "\tdeclare \@ResultSet table\n\t(\t\t@{$resultsTable->[0]} \n\t)" } ;
-#                $receivingTable = do { local $"= "\n\t,\t\t" ; "@{$resultsTable->[0]}" } ;
-            }
-            #elsif (scalar @$resultsTable gt 1 ) {
-            #    $receivingTable = "More than one set of results - can't capture them" } ;
-            #}
-#warn Dumper $receivingTable ;         
-#warn Dumper $$exec[2];
-        } ;
-
-        my $text = Template($dbh, $dbh_typeinfo, $$exec[0], $userName, $date, $$exec[1],$$exec[2],$checkText,$receivingTable ) ;
-        (my $fileName = "${file}.sql" ) =~ s{[.]}{_} ;
+        (my $fileName = "${ofile}" ) =~ s{[.]}{_} ;
         $fileName =~ s{[\]\[]}{}g ;
         $fileName =~ s{\s}{}g ;
-        my $fh   = IO::File->new("> ${dirs}/${fileName}") ;
-       
-        if (defined ${fh} ) {
-            print {${fh}} $text ;
-            $fh->close;
-        }
-        else {
-            croak "Unable to write to ${file}.sql.";
+        my $testName = $fileName;
+
+        # if not already defined in the test file (if given)
+        if ( (firstidx { $_ eq $testName } @existingTests ) == -1 ) {
+
+
+            my $checkText    = "";
+            my $receivingTable = "" ; 
+    
+            if ( $runChecks ) {
+            
+                $checkText    = CheckForExceptions($dbh, $dbh_typeinfo, $$exec[0], $userName, $date, $$exec[1],$$exec[2] ) ;
+    
+                my $resultsTable = undef ;
+                if ( ! defined $checkText || $checkText eq q() ) {
+                    $resultsTable = CheckForResults($dbh, $dbh_typeinfo, $$exec[0], $userName, $date, $$exec[1],$$exec[2] ) ;
+                }
+    #warn Dumper "--------------------------";            
+    #warn Dumper $resultsTable;
+    #warn Dumper scalar @$resultsTable ;
+    #warn Dumper @{$resultsTable->[0]};
+                if (defined $resultsTable && scalar @$resultsTable eq 1  && scalar @{$resultsTable->[0]} gt 0 ) {
+                    $receivingTable = do { local $"= "\n\t,\t\t" ; "\tdeclare \@ResultSet table\n\t(\t\t@{$resultsTable->[0]} \n\t)" } ;
+    #                $receivingTable = do { local $"= "\n\t,\t\t" ; "@{$resultsTable->[0]}" } ;
+                }
+                #elsif (scalar @$resultsTable gt 1 ) {
+                #    $receivingTable = "More than one set of results - can't capture them" } ;
+                #}
+    #warn Dumper $receivingTable ;         
+    #warn Dumper $$exec[2];
+            } ;
+    
+            my $text = Template($dbh, $dbh_typeinfo, $$exec[0], $userName, $date, $$exec[1],$$exec[2],$checkText,$receivingTable ) ;
+            $fileName .= ".sql";
+        
+            my $fh   = IO::File->new("> ${dirs}/${fileName}") ;
+        
+            if (defined ${fh} ) {
+                print {${fh}} $text ;
+                $fh->close;
+            }
+            else {
+                croak "Unable to write to ${ofile}.sql.";
+            }
         }
     }
 
